@@ -27,7 +27,7 @@ warnings.filterwarnings("ignore", message="numpy.ufunc size changed")
 
 def mkstatstr(n=None):
     """ This returns the stat structure schema or an instance of the stat structure."""
-    dtype = np.dtype([('jobid',np.str,20),('name',np.str,100),('user',np.str,100),('timeuse',np.str,100),('status',np.str,10),('queue',np.str,20)])
+    dtype = np.dtype([('jobid',str,20),('name',str,100),('user',str,100),('timeuse',str,100),('status',str,10),('queue',str,20)])
     if n is None:
         return dtype
     else:
@@ -37,8 +37,8 @@ def mkstatstr(n=None):
 
 def mkjobstr(n=None):
     """ This returns the job structure schema or an instance of the job structure."""
-    dtype = np.dtype([('host',np.str,20),('jobid',(np.str,100)),('input',np.str,1000),('dir',np.str,500),('name',np.str,100),('scriptname',np.str,200),
-                      ('logfile',np.str),('submitted',np.bool),('done',np.bool),('begtime',np.float64),('endtime',np.float64),('duration',float)])
+    dtype = np.dtype([('host',str,20),('jobid',(str,100)),('input',str,1000),('dir',str,500),('name',str,100),('scriptname',str,200),
+                      ('logfile',str),('submitted',bool),('done',bool),('begtime',float),('endtime',float),('duration',float)])
     if n is None:
         return dtype
     else:
@@ -118,9 +118,9 @@ def check_killfile(jobs=None,hyperthread=True):
             else:
                 # Using negative PID will also kill subprocesses
                 try:
-                    out = asubprocess.check_output(['kill','-9','-'+jobs['jobid'][sub[i]]],stderr=subprocess.STDOUT,shell=False)
+                    out = subprocess.check_output(['kill','-9','-'+jobs['jobid'][sub[i]]],stderr=subprocess.STDOUT,shell=False)
                 except:
-                    out = asubprocess.check_output(['kill','-9',jobs['jobid'][sub[i]]],stderr=subprocess.STDOUT,shell=False)                    
+                    out = subprocess.check_output(['kill','-9',jobs['jobid'][sub[i]]],stderr=subprocess.STDOUT,shell=False)                    
         # Remove the kill file
         print('Deleting kill file "'+killfile+'"')
         os.remove(killfile)
@@ -172,9 +172,18 @@ def makescript(inp=None,indir=None,name=None,prefix=None,hyperthread=True,idle=F
     if (inp is None):
         raise ValueError('No input given')
 
-    ninp = dln.size(inp)
-    ndir = dln.size(indir)
-    nname = dln.size(name)
+    if isinstance(inp,str):
+        ninp = 1
+    else:
+        ninp = dln.size(inp)
+    if isinstance(indir,str):
+        ndir = 1
+    else:
+        ndir = dln.size(indir)
+    if isinstance(name,str):
+        nname =1 
+    else:
+        nname = dln.size(name)
 
     # Not enough directories input
     if (ndir>0) & (ndir!=ninp):
@@ -189,7 +198,7 @@ def makescript(inp=None,indir=None,name=None,prefix=None,hyperthread=True,idle=F
 
     # Construct names
     if (nname==0):
-        name = np.zeros(ninp,dtype=(np.str,200))
+        name = np.zeros(ninp,dtype=(str,200))
         if prefix is not None:
             pre = dln.first_el(prefix)
         else:
@@ -271,7 +280,7 @@ def makescript(inp=None,indir=None,name=None,prefix=None,hyperthread=True,idle=F
             # IDL files should end in .batch
             if idle is True: scriptname[i] = str(indir[i])+'/'+base+'.batch'
             # Writing the file
-            dln.writelines(scriptname[i],cmd,overwrite=True,raw=True)
+            dln.writelines(scriptname[i],[cmd],overwrite=True)
             # Make SHELL scripts executable
             if idle is False: os.chmod(scriptname[i],0o755)
             # Print info
@@ -383,13 +392,17 @@ def getstat(jobid=None,hyperthread=True):
 
     """
     if jobid is None: raise ValueError("Must input jobid")
-    njobid = dln.size(jobid)
+    if isinstance(jobid,str) or isinstance(jobid,int):
+        njobid = 1
+    else:
+        njobid = dln.size(jobid)
+        jobid = jobid[0]
 
     # PBS
     #--------
     if hyperthread is False:
         if njobid>0: 
-            out = subprocess.check_output(['qstat',jobid[0]],stderr=subprocess.STDOUT,shell=False)
+            out = subprocess.check_output(['qstat',jobid],stderr=subprocess.STDOUT,shell=False)
         else:
             out = subprocess.check_output(['qstat'],stderr=subprocess.STDOUT,shell=False)
         nout = np.sum(out.strip() != '')
@@ -424,7 +437,7 @@ def getstat(jobid=None,hyperthread=True):
             return mkstatstr(1)
 
         try:
-            out = subprocess.check_output(['ps','-o','pid,user,etime,command','-p',str(dln.first_el(jobid))],
+            out = subprocess.check_output(['ps','-o','pid,user,etime,command','-p',str(jobid)],
                                           stderr=subprocess.STDOUT,shell=False)
         except:
             statstr = mkstatstr(1)
@@ -456,6 +469,7 @@ def getstat(jobid=None,hyperthread=True):
             statstr = mkstatstr(1)
             statstr['jobid'] = jobid
             statstr['queue'] = 'hyperthread'
+
     return statstr
 
 
@@ -495,7 +509,7 @@ def status_update(jobs=None):
 
 
 def job_daemon(inp=None,dirs=None,inpname=None,nmulti=4,prefix="job",hyperthread=True,idle=False,
-               waittime=0.2,statustime=60):
+               waittime=0.2,statustime=60,staggertime=0):
     """This program is a simple batch job manager
 
     NOTE:  If you want to "kill" all of the jobs create a file "killjobs"
@@ -527,6 +541,8 @@ def job_daemon(inp=None,dirs=None,inpname=None,nmulti=4,prefix="job",hyperthread
            Default is 60.
     waittime : float or int, optional
            Time to wait between checking the running jobs.  Default is 0.2 sec.
+    staggertime : float or int, optional
+           The time to wait between submitting jobs.  Default is 0 sec.
 
     Results
     -------
@@ -671,7 +687,9 @@ def job_daemon(inp=None,dirs=None,inpname=None,nmulti=4,prefix="job",hyperthread
                 jobs['scriptname'][newind[i]] = scriptname
                 jobs['logfile'][newind[i]] = logfile
                 jobs['begtime'][newind[i]] = time.time()/3600/24   # in days
-
+                if staggertime > 0:
+                    time.sleep(staggertime)
+                
         # Are we done?
         #-------------
         ndone = np.sum(jobs['done']==True)
