@@ -16,7 +16,8 @@ import matplotlib
 import matplotlib.pyplot as plt
 import matplotlib.colors as colors
 from matplotlib.backend_bases import MouseButton
-from scipy import stats
+#from scipy.stats import binned_statistic_2d
+from .bindata import binned_statistic_2d
 import copy
 from . import utils as dln
 from . import ladfit
@@ -129,7 +130,8 @@ def hist2d(x,y,z=None,statistic=None,xr=None,yr=None,dx=None,dy=None,nx=200,ny=2
            xflip=False,yflip=False,force=True,cmap=None,figure=None,
            figsize=(8,8),xtitle=None,ytitle=None,title=None,colorlabel=None,charsize=12,
            origin='lower',aspect='auto',interpolation='none',bright=None,minhue=0.0,
-           maxhue=0.7,minbright=0.1,maxbright=0.7,saturation=0.9,noplot=False,save=None):
+           maxhue=0.7,minbright=0.1,maxbright=0.7,saturation=0.9,noplot=False,save=None,
+           colorbar=True):
     """
     Make a 2D histogram of points.
     
@@ -143,7 +145,8 @@ def hist2d(x,y,z=None,statistic=None,xr=None,yr=None,dx=None,dy=None,nx=200,ny=2
        Array of z-data points to plot if we using a statistic.
     statistic : str, optional
        The statistic to use.  Default is 'count' if z is not input otherwise 'sum'.
-         The options are: 'sum', 'mean', 'median', 'std', 'mad'.
+         The options are: 'sum', 'mean', 'median', 'std', 'mad'.  A function can
+         also be input to compute the statistic.
     xr : list, optional
        The range of x-values to plot.  The default is [min(x),max(x)]
     yr : list, optional
@@ -207,12 +210,21 @@ def hist2d(x,y,z=None,statistic=None,xr=None,yr=None,dx=None,dy=None,nx=200,ny=2
        Do not plot anything.  Just return the data.
     save : str, optional
        Save the figure to this file.
+    colorbar : bool, optional
+       Shows a colorbar.  Default is True.
 
     Returns
     -------
+    xedges : numpy array
+       Array of edges in the X-direction.
+    yedges : numpy array
+       Array of edges in the Y-direction.
+    result : numpy array
+       The resulting 2-D numpy array.
 
     Figure is plotted to the screen.
 
+
     Example
     -------
 
@@ -424,17 +436,17 @@ def hist2d(x,y,z=None,statistic=None,xr=None,yr=None,dx=None,dy=None,nx=200,ny=2
     else:
         if statistic=='avg':
             cmap = 'jet'
-            ima, xedges, yedges, binnumber = stats.binned_statistic_2d(x,y,z,statistic='mean',range=[xr,yr],bins=[nx,ny])
+            ima, xedges, yedges, binnumber = binned_statistic_2d(x,y,z,statistic='mean',range=[xr,yr],bins=[nx,ny])
             ima = ima.T
             if bright is None:
-                imt, xedges, yedges, binnumber = stats.binned_statistic_2d(x,y,z,statistic='count',range=[xr,yr],bins=[nx,ny])
+                imt, xedges, yedges, binnumber = binned_statistic_2d(x,y,z,statistic='count',range=[xr,yr],bins=[nx,ny])
                 imt = imt.T
                 if log:
                     pos = (imt > 0)
                     imt[pos] = np.log10(imt[pos])
                     imt[~pos] = -10
             else:
-                imt, xedges, yedges, binnumber = stats.binned_statistic_2d(x,y,bright,statistic='sum',range=[xr,yr],bins=[nx,ny])
+                imt, xedges, yedges, binnumber = binned_statistic_2d(x,y,bright,statistic='sum',range=[xr,yr],bins=[nx,ny])
                 imt = imt.T
                 if log:
                     pos = (imt > 0)
@@ -480,7 +492,7 @@ def hist2d(x,y,z=None,statistic=None,xr=None,yr=None,dx=None,dy=None,nx=200,ny=2
             
         # Normal statistic
         else:
-            im, xedges, yedges, binnumber = stats.binned_statistic_2d(x,y,z,statistic=statistic,range=[xr,yr],bins=[nx,ny])
+            im, xedges, yedges, binnumber = binned_statistic_2d(x,y,z,statistic=statistic,range=[xr,yr],bins=[nx,ny])
             im = im.T
 
     # Plot the image
@@ -512,19 +524,26 @@ def hist2d(x,y,z=None,statistic=None,xr=None,yr=None,dx=None,dy=None,nx=200,ny=2
             ytitle = 'Y'
         plt.ylabel(ytitle)        
         if title is None:
-            if statistic is 'count':
+            if statistic == 'count':
                 title = statistic
             else:
-                title = statistic+'(Z)'
+                if isinstance(statistic,str):
+                    title = statistic+'(Z)'
+                else:
+                    title = 'function(Z)'
         plt.title(title)
         
         # Add the colorbar
         if colorlabel is None:
-            if statistic is 'count':
+            if statistic == 'count':
                 colorlabel = statistic
             else:
-                colorlabel = statistic+'(Z)'
-        plt.colorbar(label=colorlabel)
+                if isinstance(statistic,str):
+                    colorlabel = statistic+'(Z)'
+                else:
+                    colorlabel = 'function(Z)'
+        if colorbar:
+            plt.colorbar(label=colorlabel)
 
         # Save the figure
         if save is not None:
@@ -536,7 +555,7 @@ def hist2d(x,y,z=None,statistic=None,xr=None,yr=None,dx=None,dy=None,nx=200,ny=2
 def display(im,x=None,y=None,log=False,xr=None,yr=None,noerase=False,zscale=False,norm=None,
             vmin=None,vmax=None,xtitle=None,ytitle=None,title=None,origin='lower',aspect='auto',
             xflip=False,yflip=False,cmap=None,figure=None,figsize=(8,8),save=None,colorlabel=None,
-            charsize=12,interpolation='none'):
+            charsize=12,interpolation='none',colorbar=True):
     """
     Display an image.  The usual python convention is used where the image is [NY,NX]
 
@@ -595,6 +614,8 @@ def display(im,x=None,y=None,log=False,xr=None,yr=None,noerase=False,zscale=Fals
        Character size.  Default is 12.
     interpolation : str, optional
        The type of interpolation.  Default is 'none'.
+    colorbar : bool, optional
+       Show a colorbar.  Default is True.
 
     Returns
     -------
@@ -632,8 +653,10 @@ def display(im,x=None,y=None,log=False,xr=None,yr=None,noerase=False,zscale=Fals
     # Min's and Max's
     xmin = np.min(x)
     xmax = np.max(x)
+    dx = x[1]-x[0]
     ymin = np.min(y)
     ymax = np.max(y)
+    dy = y[1]-y[0]
 
     # Trim based on xrange/yrange
     if xr is not None:
@@ -644,6 +667,7 @@ def display(im,x=None,y=None,log=False,xr=None,yr=None,noerase=False,zscale=Fals
         nx,ny = im.shape
         xmin = np.min(x)
         xmax = np.max(x)
+        dx = x[1]-x[0]
     if yr is not None:
         val0,ylo = dln.closest(y,yr[0])
         val1,yhi = dln.closest(y,yr[1])        
@@ -652,6 +676,7 @@ def display(im,x=None,y=None,log=False,xr=None,yr=None,noerase=False,zscale=Fals
         nx,ny = im.shape        
         ymin = np.min(y)
         ymax = np.max(y)
+        dy = y[1]-y[0]
         
     # Plot the image
     #fig, ax = plt.subplots()
@@ -661,7 +686,9 @@ def display(im,x=None,y=None,log=False,xr=None,yr=None,noerase=False,zscale=Fals
         vmin,vmax = zscaling(im)
 
     # (left, right, bottom, top)
-    extent = [xmin, xmax, ymin, ymax]
+    #extent = [xmin, xmax, ymin, ymax]
+    extent = [xmin-0.5*dx, xmax+0.5*dx,  # 1/2 pixel offset
+              ymin-0.5*dy, ymax+0.5*dy]    
     if norm is not None:
         plt.imshow(im,cmap=cmap,norm=norm,aspect=aspect,origin=origin,
                    extent=extent,interpolation=interpolation)
@@ -690,7 +717,8 @@ def display(im,x=None,y=None,log=False,xr=None,yr=None,noerase=False,zscale=Fals
     # Add the colorbar
     if colorlabel is None:
         colorlabel = ''
-    plt.colorbar(label=colorlabel)
+    if colorbar:
+        plt.colorbar(label=colorlabel)
 
     # Save the figure
     if save is not None:
@@ -809,9 +837,9 @@ def plot(x,y=None,c=None,fmt=None,marker=None,fill=True,size=None,log=False,noer
     # Line plot
     if (c is None or type(c) is str) and marker is None:
         if fmt is not None:
-            plt.plot(x,y,fmt,c=c)
+            plt.plot(x,y,fmt,c=c,linewidth=linewidth)
         else:
-            plt.plot(x,y,c=c)
+            plt.plot(x,y,c=c,linewidth=linewidth)
         
     # Scatter plot
     else:
@@ -873,6 +901,44 @@ def oplot(*args,**kwargs):
 def scatter(*args,marker='o',**kwargs):
     plot(*args,marker=marker,**kwargs)
 
+def hist(*args,bins=20,figure=None,figsize=(8,8),noerase=False,charsize=12,**kwargs):
+
+    # Getting the current figure, creating a new one if necessary
+    if figure is None:
+        fig = plt.gcf()
+    else:
+        fig = plt.figure(figure,figsize=figsize)
+
+    # Character size
+    font = {'size': charsize}
+    matplotlib.rc('font', **font)
+        
+    if noerase is False:
+        plt.clf()   # clear the plot
+
+    xtitle,ytitle,title = None,None,None
+    if 'xtitle' in kwargs:
+        xtitle = kwargs['xtitle']
+        del kwargs['xtitle']
+    if 'ytitle' in kwargs:
+        ytitle = kwargs['ytitle']
+        del kwargs['ytitle']        
+    if 'title' in kwargs:
+        title = kwargs['title']        
+        del kwargs['title']
+
+    # Run regular matplotlib hist()
+    out = plt.hist(*args,bins=bins,**kwargs)
+
+    # titles
+    if xtitle is not None:
+        plt.xlabel(xtitle)
+    if ytitle is not None:
+        plt.ylabel(ytitle)
+    if title is not None:
+        plt.title(title)        
+    
+    return out
 
 class Cursor():
 
@@ -1016,7 +1082,7 @@ def curpdiff(spherical=False,arcsec=False):
                 print('Angle = %.4f %s' % (angle,'(CCW from Right)'))
                 slp,yoff = dln.linear_coefficients([x1,x2],[y1,y2])
                 if yoff >= 0:
-                    print('y = %.3f*x - %.3f' % (slp,yoff))                    
+                    print('y = %.3f*x + %.3f' % (slp,yoff))                    
                 else:
                     print('y = %.3f*x - %.3f' % (slp,abs(yoff)))
                           
